#!/usr/bin/python3.9
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#
#
# Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#

from __future__ import print_function
import pkg.fmri
import sys

def usage():
        print("usage: %s <fmri1> <fmri2>".format(sys.argv[0]), file=sys.stderr)
        sys.exit(2)

if len(sys.argv) != 3:
        usage()

try:
        x = pkg.fmri.PkgFmri(sys.argv[1])
        y = pkg.fmri.PkgFmri(sys.argv[2])
except pkg.fmri.FmriError as e:
        print ("error: %s" % str(e) , file=sys.stderr)
        sys.exit(1)

if not x.is_same_pkg(y):
        print ("error: can only compare two versions of the same package.",
            file=sys.stderr)
        sys.exit(1)

if x < y:
        print("<")
elif x > y:
        print(">")
elif x == y:
        print("=")
else:
        print ("panic", file=sys.stderr)
        sys.exit(1)

sys.exit(0)

# Vim hints
# vim:ts=8:sw=8:et:fdm=marker
