#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
# define _cffi_float_complex_t   _Fcomplex    /* include <complex.h> for it */
# define _cffi_double_complex_t  _Dcomplex    /* include <complex.h> for it */
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
# define _cffi_float_complex_t   float _Complex
# define _cffi_double_complex_t  double _Complex
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


/* Includes */
#include <attr.h>
#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <sys/nvpair.h>


/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 43), // char *()(nvpair_t *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 49), // nvpair_t *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // char const *()(f_attr_t)
/*  4 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // f_attr_t
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // data_type_t()(nvpair_t *)
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION, 4), // f_attr_t()(char const *)
/* 10 */ _CFFI_OP(_CFFI_OP_POINTER, 44), // char const *
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // int()(int, xattr_view_t, nvlist_t * *)
/* 13 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 14 */ _CFFI_OP(_CFFI_OP_ENUM, 3), // xattr_view_t
/* 15 */ _CFFI_OP(_CFFI_OP_POINTER, 20), // nvlist_t * *
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // int()(int, xattr_view_t, nvlist_t *)
/* 18 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 19 */ _CFFI_OP(_CFFI_OP_NOOP, 14),
/* 20 */ _CFFI_OP(_CFFI_OP_POINTER, 48), // nvlist_t *
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // int()(nvlist_t * *, unsigned int, int)
/* 23 */ _CFFI_OP(_CFFI_OP_NOOP, 15),
/* 24 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 25 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // int()(nvlist_t *, char const *, boolean_t)
/* 28 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 29 */ _CFFI_OP(_CFFI_OP_NOOP, 10),
/* 30 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // boolean_t
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION, 13), // int()(nvpair_t *, boolean_t *)
/* 33 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 34 */ _CFFI_OP(_CFFI_OP_POINTER, 30), // boolean_t *
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // nvpair_t *()(nvlist_t *, nvpair_t *)
/* 37 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION, 52), // void()(nvlist_t *)
/* 41 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_POINTER, 44), // char *
/* 44 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 45 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // data_type_t
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 19), // int16_t
/* 47 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 21), // int32_t
/* 48 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // nvlist_t
/* 49 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // nvpair_t
/* 50 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22), // uint32_t
/* 51 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24), // uint64_t
/* 52 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const__B_FALSE(unsigned long long *o)
{
  int n = (_B_FALSE) <= 0;
  *o = (unsigned long long)((_B_FALSE) | 0);  /* check that _B_FALSE is an integer */
  return n;
}

static int _cffi_const__B_TRUE(unsigned long long *o)
{
  int n = (_B_TRUE) <= 0;
  *o = (unsigned long long)((_B_TRUE) | 0);  /* check that _B_TRUE is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UNKNOWN(unsigned long long *o)
{
  int n = (DATA_TYPE_UNKNOWN) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UNKNOWN) | 0);  /* check that DATA_TYPE_UNKNOWN is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_BOOLEAN(unsigned long long *o)
{
  int n = (DATA_TYPE_BOOLEAN) <= 0;
  *o = (unsigned long long)((DATA_TYPE_BOOLEAN) | 0);  /* check that DATA_TYPE_BOOLEAN is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_BYTE(unsigned long long *o)
{
  int n = (DATA_TYPE_BYTE) <= 0;
  *o = (unsigned long long)((DATA_TYPE_BYTE) | 0);  /* check that DATA_TYPE_BYTE is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT16(unsigned long long *o)
{
  int n = (DATA_TYPE_INT16) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT16) | 0);  /* check that DATA_TYPE_INT16 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT16(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT16) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT16) | 0);  /* check that DATA_TYPE_UINT16 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT32(unsigned long long *o)
{
  int n = (DATA_TYPE_INT32) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT32) | 0);  /* check that DATA_TYPE_INT32 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT32(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT32) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT32) | 0);  /* check that DATA_TYPE_UINT32 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT64(unsigned long long *o)
{
  int n = (DATA_TYPE_INT64) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT64) | 0);  /* check that DATA_TYPE_INT64 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT64(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT64) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT64) | 0);  /* check that DATA_TYPE_UINT64 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_STRING(unsigned long long *o)
{
  int n = (DATA_TYPE_STRING) <= 0;
  *o = (unsigned long long)((DATA_TYPE_STRING) | 0);  /* check that DATA_TYPE_STRING is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_BYTE_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_BYTE_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_BYTE_ARRAY) | 0);  /* check that DATA_TYPE_BYTE_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT16_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_INT16_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT16_ARRAY) | 0);  /* check that DATA_TYPE_INT16_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT16_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT16_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT16_ARRAY) | 0);  /* check that DATA_TYPE_UINT16_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT32_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_INT32_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT32_ARRAY) | 0);  /* check that DATA_TYPE_INT32_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT32_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT32_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT32_ARRAY) | 0);  /* check that DATA_TYPE_UINT32_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT64_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_INT64_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT64_ARRAY) | 0);  /* check that DATA_TYPE_INT64_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT64_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT64_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT64_ARRAY) | 0);  /* check that DATA_TYPE_UINT64_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_STRING_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_STRING_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_STRING_ARRAY) | 0);  /* check that DATA_TYPE_STRING_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_HRTIME(unsigned long long *o)
{
  int n = (DATA_TYPE_HRTIME) <= 0;
  *o = (unsigned long long)((DATA_TYPE_HRTIME) | 0);  /* check that DATA_TYPE_HRTIME is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_NVLIST(unsigned long long *o)
{
  int n = (DATA_TYPE_NVLIST) <= 0;
  *o = (unsigned long long)((DATA_TYPE_NVLIST) | 0);  /* check that DATA_TYPE_NVLIST is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_NVLIST_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_NVLIST_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_NVLIST_ARRAY) | 0);  /* check that DATA_TYPE_NVLIST_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_BOOLEAN_VALUE(unsigned long long *o)
{
  int n = (DATA_TYPE_BOOLEAN_VALUE) <= 0;
  *o = (unsigned long long)((DATA_TYPE_BOOLEAN_VALUE) | 0);  /* check that DATA_TYPE_BOOLEAN_VALUE is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT8(unsigned long long *o)
{
  int n = (DATA_TYPE_INT8) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT8) | 0);  /* check that DATA_TYPE_INT8 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT8(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT8) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT8) | 0);  /* check that DATA_TYPE_UINT8 is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_BOOLEAN_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_BOOLEAN_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_BOOLEAN_ARRAY) | 0);  /* check that DATA_TYPE_BOOLEAN_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_INT8_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_INT8_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_INT8_ARRAY) | 0);  /* check that DATA_TYPE_INT8_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_UINT8_ARRAY(unsigned long long *o)
{
  int n = (DATA_TYPE_UINT8_ARRAY) <= 0;
  *o = (unsigned long long)((DATA_TYPE_UINT8_ARRAY) | 0);  /* check that DATA_TYPE_UINT8_ARRAY is an integer */
  return n;
}

static int _cffi_const_DATA_TYPE_DOUBLE(unsigned long long *o)
{
  int n = (DATA_TYPE_DOUBLE) <= 0;
  *o = (unsigned long long)((DATA_TYPE_DOUBLE) | 0);  /* check that DATA_TYPE_DOUBLE is an integer */
  return n;
}

static int _cffi_const_F_ATTR_INVAL(unsigned long long *o)
{
  int n = (F_ATTR_INVAL) <= 0;
  *o = (unsigned long long)((F_ATTR_INVAL) | 0);  /* check that F_ATTR_INVAL is an integer */
  return n;
}

static int _cffi_const_F_ARCHIVE(unsigned long long *o)
{
  int n = (F_ARCHIVE) <= 0;
  *o = (unsigned long long)((F_ARCHIVE) | 0);  /* check that F_ARCHIVE is an integer */
  return n;
}

static int _cffi_const_F_HIDDEN(unsigned long long *o)
{
  int n = (F_HIDDEN) <= 0;
  *o = (unsigned long long)((F_HIDDEN) | 0);  /* check that F_HIDDEN is an integer */
  return n;
}

static int _cffi_const_F_READONLY(unsigned long long *o)
{
  int n = (F_READONLY) <= 0;
  *o = (unsigned long long)((F_READONLY) | 0);  /* check that F_READONLY is an integer */
  return n;
}

static int _cffi_const_F_SYSTEM(unsigned long long *o)
{
  int n = (F_SYSTEM) <= 0;
  *o = (unsigned long long)((F_SYSTEM) | 0);  /* check that F_SYSTEM is an integer */
  return n;
}

static int _cffi_const_F_APPENDONLY(unsigned long long *o)
{
  int n = (F_APPENDONLY) <= 0;
  *o = (unsigned long long)((F_APPENDONLY) | 0);  /* check that F_APPENDONLY is an integer */
  return n;
}

static int _cffi_const_F_NODUMP(unsigned long long *o)
{
  int n = (F_NODUMP) <= 0;
  *o = (unsigned long long)((F_NODUMP) | 0);  /* check that F_NODUMP is an integer */
  return n;
}

static int _cffi_const_F_IMMUTABLE(unsigned long long *o)
{
  int n = (F_IMMUTABLE) <= 0;
  *o = (unsigned long long)((F_IMMUTABLE) | 0);  /* check that F_IMMUTABLE is an integer */
  return n;
}

static int _cffi_const_F_AV_MODIFIED(unsigned long long *o)
{
  int n = (F_AV_MODIFIED) <= 0;
  *o = (unsigned long long)((F_AV_MODIFIED) | 0);  /* check that F_AV_MODIFIED is an integer */
  return n;
}

static int _cffi_const_F_OPAQUE(unsigned long long *o)
{
  int n = (F_OPAQUE) <= 0;
  *o = (unsigned long long)((F_OPAQUE) | 0);  /* check that F_OPAQUE is an integer */
  return n;
}

static int _cffi_const_F_AV_SCANSTAMP(unsigned long long *o)
{
  int n = (F_AV_SCANSTAMP) <= 0;
  *o = (unsigned long long)((F_AV_SCANSTAMP) | 0);  /* check that F_AV_SCANSTAMP is an integer */
  return n;
}

static int _cffi_const_F_AV_QUARANTINED(unsigned long long *o)
{
  int n = (F_AV_QUARANTINED) <= 0;
  *o = (unsigned long long)((F_AV_QUARANTINED) | 0);  /* check that F_AV_QUARANTINED is an integer */
  return n;
}

static int _cffi_const_F_NOUNLINK(unsigned long long *o)
{
  int n = (F_NOUNLINK) <= 0;
  *o = (unsigned long long)((F_NOUNLINK) | 0);  /* check that F_NOUNLINK is an integer */
  return n;
}

static int _cffi_const_F_CRTIME(unsigned long long *o)
{
  int n = (F_CRTIME) <= 0;
  *o = (unsigned long long)((F_CRTIME) | 0);  /* check that F_CRTIME is an integer */
  return n;
}

static int _cffi_const_F_OWNERSID(unsigned long long *o)
{
  int n = (F_OWNERSID) <= 0;
  *o = (unsigned long long)((F_OWNERSID) | 0);  /* check that F_OWNERSID is an integer */
  return n;
}

static int _cffi_const_F_GROUPSID(unsigned long long *o)
{
  int n = (F_GROUPSID) <= 0;
  *o = (unsigned long long)((F_GROUPSID) | 0);  /* check that F_GROUPSID is an integer */
  return n;
}

static int _cffi_const_F_FSID(unsigned long long *o)
{
  int n = (F_FSID) <= 0;
  *o = (unsigned long long)((F_FSID) | 0);  /* check that F_FSID is an integer */
  return n;
}

static int _cffi_const_F_REPARSE(unsigned long long *o)
{
  int n = (F_REPARSE) <= 0;
  *o = (unsigned long long)((F_REPARSE) | 0);  /* check that F_REPARSE is an integer */
  return n;
}

static int _cffi_const_F_GEN(unsigned long long *o)
{
  int n = (F_GEN) <= 0;
  *o = (unsigned long long)((F_GEN) | 0);  /* check that F_GEN is an integer */
  return n;
}

static int _cffi_const_F_OFFLINE(unsigned long long *o)
{
  int n = (F_OFFLINE) <= 0;
  *o = (unsigned long long)((F_OFFLINE) | 0);  /* check that F_OFFLINE is an integer */
  return n;
}

static int _cffi_const_F_SPARSE(unsigned long long *o)
{
  int n = (F_SPARSE) <= 0;
  *o = (unsigned long long)((F_SPARSE) | 0);  /* check that F_SPARSE is an integer */
  return n;
}

static int _cffi_const_F_ATTR_ALL(unsigned long long *o)
{
  int n = (F_ATTR_ALL) <= 0;
  *o = (unsigned long long)((F_ATTR_ALL) | 0);  /* check that F_ATTR_ALL is an integer */
  return n;
}

static int _cffi_const_XATTR_VIEW_INVALID(unsigned long long *o)
{
  int n = (XATTR_VIEW_INVALID) <= 0;
  *o = (unsigned long long)((XATTR_VIEW_INVALID) | 0);  /* check that XATTR_VIEW_INVALID is an integer */
  return n;
}

static int _cffi_const_XATTR_VIEW_READONLY(unsigned long long *o)
{
  int n = (XATTR_VIEW_READONLY) <= 0;
  *o = (unsigned long long)((XATTR_VIEW_READONLY) | 0);  /* check that XATTR_VIEW_READONLY is an integer */
  return n;
}

static int _cffi_const_XATTR_VIEW_READWRITE(unsigned long long *o)
{
  int n = (XATTR_VIEW_READWRITE) <= 0;
  *o = (unsigned long long)((XATTR_VIEW_READWRITE) | 0);  /* check that XATTR_VIEW_READWRITE is an integer */
  return n;
}

static int _cffi_const_XATTR_VIEW_LAST(unsigned long long *o)
{
  int n = (XATTR_VIEW_LAST) <= 0;
  *o = (unsigned long long)((XATTR_VIEW_LAST) | 0);  /* check that XATTR_VIEW_LAST is an integer */
  return n;
}

static char const * _cffi_d_attr_to_name(f_attr_t x0)
{
  return attr_to_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_attr_to_name(PyObject *self, PyObject *arg0)
{
  f_attr_t x0;
  char const * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(4), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = attr_to_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(10));
  return pyresult;
}
#else
#  define _cffi_f_attr_to_name _cffi_d_attr_to_name
#endif

static char const * _cffi_d_attr_to_option(f_attr_t x0)
{
  return attr_to_option(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_attr_to_option(PyObject *self, PyObject *arg0)
{
  f_attr_t x0;
  char const * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(4), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = attr_to_option(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(10));
  return pyresult;
}
#else
#  define _cffi_f_attr_to_option _cffi_d_attr_to_option
#endif

static int _cffi_d_fgetattr(int x0, xattr_view_t x1, nvlist_t * * x2)
{
  return fgetattr(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_fgetattr(PyObject *self, PyObject *args)
{
  int x0;
  xattr_view_t x1;
  nvlist_t * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "fgetattr", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(14), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (nvlist_t * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = fgetattr(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_fgetattr _cffi_d_fgetattr
#endif

static int _cffi_d_fsetattr(int x0, xattr_view_t x1, nvlist_t * x2)
{
  return fsetattr(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_fsetattr(PyObject *self, PyObject *args)
{
  int x0;
  xattr_view_t x1;
  nvlist_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "fsetattr", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(14), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (nvlist_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = fsetattr(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_fsetattr _cffi_d_fsetattr
#endif

static f_attr_t _cffi_d_name_to_attr(char const * x0)
{
  return name_to_attr(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_name_to_attr(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  f_attr_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = name_to_attr(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(4));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_name_to_attr _cffi_d_name_to_attr
#endif

static int _cffi_d_nvlist_add_boolean_value(nvlist_t * x0, char const * x1, boolean_t x2)
{
  return nvlist_add_boolean_value(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvlist_add_boolean_value(PyObject *self, PyObject *args)
{
  nvlist_t * x0;
  char const * x1;
  boolean_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "nvlist_add_boolean_value", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvlist_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(30), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvlist_add_boolean_value(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvlist_add_boolean_value _cffi_d_nvlist_add_boolean_value
#endif

static int _cffi_d_nvlist_alloc(nvlist_t * * x0, unsigned int x1, int x2)
{
  return nvlist_alloc(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvlist_alloc(PyObject *self, PyObject *args)
{
  nvlist_t * * x0;
  unsigned int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "nvlist_alloc", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvlist_t * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvlist_alloc(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvlist_alloc _cffi_d_nvlist_alloc
#endif

static void _cffi_d_nvlist_free(nvlist_t * x0)
{
  nvlist_free(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvlist_free(PyObject *self, PyObject *arg0)
{
  nvlist_t * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvlist_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { nvlist_free(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_nvlist_free _cffi_d_nvlist_free
#endif

static nvpair_t * _cffi_d_nvlist_next_nvpair(nvlist_t * x0, nvpair_t * x1)
{
  return nvlist_next_nvpair(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvlist_next_nvpair(PyObject *self, PyObject *args)
{
  nvlist_t * x0;
  nvpair_t * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  nvpair_t * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "nvlist_next_nvpair", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvlist_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (nvpair_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvlist_next_nvpair(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvlist_next_nvpair _cffi_d_nvlist_next_nvpair
#endif

static char * _cffi_d_nvpair_name(nvpair_t * x0)
{
  return nvpair_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvpair_name(PyObject *self, PyObject *arg0)
{
  nvpair_t * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvpair_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvpair_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(43));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvpair_name _cffi_d_nvpair_name
#endif

static data_type_t _cffi_d_nvpair_type(nvpair_t * x0)
{
  return nvpair_type(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvpair_type(PyObject *self, PyObject *arg0)
{
  nvpair_t * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  data_type_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvpair_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvpair_type(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvpair_type _cffi_d_nvpair_type
#endif

static int _cffi_d_nvpair_value_boolean_value(nvpair_t * x0, boolean_t * x1)
{
  return nvpair_value_boolean_value(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_nvpair_value_boolean_value(PyObject *self, PyObject *args)
{
  nvpair_t * x0;
  boolean_t * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "nvpair_value_boolean_value", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (nvpair_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(34), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (boolean_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(34), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = nvpair_value_boolean_value(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_nvpair_value_boolean_value _cffi_d_nvpair_value_boolean_value
#endif

static f_attr_t _cffi_d_option_to_attr(char const * x0)
{
  return option_to_attr(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_option_to_attr(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  f_attr_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = option_to_attr(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(4));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_option_to_attr _cffi_d_option_to_attr
#endif

static int _cffi_const_NV_UNIQUE_NAME(unsigned long long *o)
{
  int n = (NV_UNIQUE_NAME) <= 0;
  *o = (unsigned long long)((NV_UNIQUE_NAME) | 0);  /* check that NV_UNIQUE_NAME is an integer */
  if (!_cffi_check_int(*o, n, 1U))
    n |= 2;
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld__nvlist_t(nvlist_t *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->nvl_version) | 0);  /* check that 'nvlist_t.nvl_version' is an integer */
  (void)((p->nvl_nvflag) | 0);  /* check that 'nvlist_t.nvl_nvflag' is an integer */
  (void)((p->nvl_priv) | 0);  /* check that 'nvlist_t.nvl_priv' is an integer */
  (void)((p->nvl_flag) | 0);  /* check that 'nvlist_t.nvl_flag' is an integer */
  (void)((p->nvl_pad) | 0);  /* check that 'nvlist_t.nvl_pad' is an integer */
}
struct _cffi_align__nvlist_t { char x; nvlist_t y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__nvpair_t(nvpair_t *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->nvp_size) | 0);  /* check that 'nvpair_t.nvp_size' is an integer */
  (void)((p->nvp_name_sz) | 0);  /* check that 'nvpair_t.nvp_name_sz' is an integer */
  (void)((p->nvp_reserve) | 0);  /* check that 'nvpair_t.nvp_reserve' is an integer */
  (void)((p->nvp_value_elem) | 0);  /* check that 'nvpair_t.nvp_value_elem' is an integer */
  { data_type_t *tmp = &p->nvp_type; (void)tmp; }
}
struct _cffi_align__nvpair_t { char x; nvpair_t y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "DATA_TYPE_BOOLEAN", (void *)_cffi_const_DATA_TYPE_BOOLEAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_BOOLEAN_ARRAY", (void *)_cffi_const_DATA_TYPE_BOOLEAN_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_BOOLEAN_VALUE", (void *)_cffi_const_DATA_TYPE_BOOLEAN_VALUE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_BYTE", (void *)_cffi_const_DATA_TYPE_BYTE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_BYTE_ARRAY", (void *)_cffi_const_DATA_TYPE_BYTE_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_DOUBLE", (void *)_cffi_const_DATA_TYPE_DOUBLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_HRTIME", (void *)_cffi_const_DATA_TYPE_HRTIME, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT16", (void *)_cffi_const_DATA_TYPE_INT16, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT16_ARRAY", (void *)_cffi_const_DATA_TYPE_INT16_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT32", (void *)_cffi_const_DATA_TYPE_INT32, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT32_ARRAY", (void *)_cffi_const_DATA_TYPE_INT32_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT64", (void *)_cffi_const_DATA_TYPE_INT64, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT64_ARRAY", (void *)_cffi_const_DATA_TYPE_INT64_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT8", (void *)_cffi_const_DATA_TYPE_INT8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_INT8_ARRAY", (void *)_cffi_const_DATA_TYPE_INT8_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_NVLIST", (void *)_cffi_const_DATA_TYPE_NVLIST, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_NVLIST_ARRAY", (void *)_cffi_const_DATA_TYPE_NVLIST_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_STRING", (void *)_cffi_const_DATA_TYPE_STRING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_STRING_ARRAY", (void *)_cffi_const_DATA_TYPE_STRING_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT16", (void *)_cffi_const_DATA_TYPE_UINT16, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT16_ARRAY", (void *)_cffi_const_DATA_TYPE_UINT16_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT32", (void *)_cffi_const_DATA_TYPE_UINT32, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT32_ARRAY", (void *)_cffi_const_DATA_TYPE_UINT32_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT64", (void *)_cffi_const_DATA_TYPE_UINT64, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT64_ARRAY", (void *)_cffi_const_DATA_TYPE_UINT64_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT8", (void *)_cffi_const_DATA_TYPE_UINT8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UINT8_ARRAY", (void *)_cffi_const_DATA_TYPE_UINT8_ARRAY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "DATA_TYPE_UNKNOWN", (void *)_cffi_const_DATA_TYPE_UNKNOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_APPENDONLY", (void *)_cffi_const_F_APPENDONLY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_ARCHIVE", (void *)_cffi_const_F_ARCHIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_ATTR_ALL", (void *)_cffi_const_F_ATTR_ALL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_ATTR_INVAL", (void *)_cffi_const_F_ATTR_INVAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_AV_MODIFIED", (void *)_cffi_const_F_AV_MODIFIED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_AV_QUARANTINED", (void *)_cffi_const_F_AV_QUARANTINED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_AV_SCANSTAMP", (void *)_cffi_const_F_AV_SCANSTAMP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_CRTIME", (void *)_cffi_const_F_CRTIME, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_FSID", (void *)_cffi_const_F_FSID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_GEN", (void *)_cffi_const_F_GEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_GROUPSID", (void *)_cffi_const_F_GROUPSID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_HIDDEN", (void *)_cffi_const_F_HIDDEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_IMMUTABLE", (void *)_cffi_const_F_IMMUTABLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_NODUMP", (void *)_cffi_const_F_NODUMP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_NOUNLINK", (void *)_cffi_const_F_NOUNLINK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_OFFLINE", (void *)_cffi_const_F_OFFLINE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_OPAQUE", (void *)_cffi_const_F_OPAQUE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_OWNERSID", (void *)_cffi_const_F_OWNERSID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_READONLY", (void *)_cffi_const_F_READONLY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_REPARSE", (void *)_cffi_const_F_REPARSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_SPARSE", (void *)_cffi_const_F_SPARSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "F_SYSTEM", (void *)_cffi_const_F_SYSTEM, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NV_UNIQUE_NAME", (void *)_cffi_const_NV_UNIQUE_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "XATTR_VIEW_INVALID", (void *)_cffi_const_XATTR_VIEW_INVALID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "XATTR_VIEW_LAST", (void *)_cffi_const_XATTR_VIEW_LAST, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "XATTR_VIEW_READONLY", (void *)_cffi_const_XATTR_VIEW_READONLY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "XATTR_VIEW_READWRITE", (void *)_cffi_const_XATTR_VIEW_READWRITE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "_B_FALSE", (void *)_cffi_const__B_FALSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "_B_TRUE", (void *)_cffi_const__B_TRUE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "attr_to_name", (void *)_cffi_f_attr_to_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 3), (void *)_cffi_d_attr_to_name },
  { "attr_to_option", (void *)_cffi_f_attr_to_option, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 3), (void *)_cffi_d_attr_to_option },
  { "fgetattr", (void *)_cffi_f_fgetattr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 12), (void *)_cffi_d_fgetattr },
  { "fsetattr", (void *)_cffi_f_fsetattr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 17), (void *)_cffi_d_fsetattr },
  { "name_to_attr", (void *)_cffi_f_name_to_attr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 9), (void *)_cffi_d_name_to_attr },
  { "nvlist_add_boolean_value", (void *)_cffi_f_nvlist_add_boolean_value, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_nvlist_add_boolean_value },
  { "nvlist_alloc", (void *)_cffi_f_nvlist_alloc, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 22), (void *)_cffi_d_nvlist_alloc },
  { "nvlist_free", (void *)_cffi_f_nvlist_free, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 40), (void *)_cffi_d_nvlist_free },
  { "nvlist_next_nvpair", (void *)_cffi_f_nvlist_next_nvpair, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 36), (void *)_cffi_d_nvlist_next_nvpair },
  { "nvpair_name", (void *)_cffi_f_nvpair_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_nvpair_name },
  { "nvpair_type", (void *)_cffi_f_nvpair_type, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 6), (void *)_cffi_d_nvpair_type },
  { "nvpair_value_boolean_value", (void *)_cffi_f_nvpair_value_boolean_value, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 32), (void *)_cffi_d_nvpair_value_boolean_value },
  { "option_to_attr", (void *)_cffi_f_option_to_attr, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 9), (void *)_cffi_d_option_to_attr },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "nvl_version", offsetof(nvlist_t, nvl_version),
                   sizeof(((nvlist_t *)0)->nvl_version),
                   _CFFI_OP(_CFFI_OP_NOOP, 47) },
  { "nvl_nvflag", offsetof(nvlist_t, nvl_nvflag),
                  sizeof(((nvlist_t *)0)->nvl_nvflag),
                  _CFFI_OP(_CFFI_OP_NOOP, 50) },
  { "nvl_priv", offsetof(nvlist_t, nvl_priv),
                sizeof(((nvlist_t *)0)->nvl_priv),
                _CFFI_OP(_CFFI_OP_NOOP, 51) },
  { "nvl_flag", offsetof(nvlist_t, nvl_flag),
                sizeof(((nvlist_t *)0)->nvl_flag),
                _CFFI_OP(_CFFI_OP_NOOP, 50) },
  { "nvl_pad", offsetof(nvlist_t, nvl_pad),
               sizeof(((nvlist_t *)0)->nvl_pad),
               _CFFI_OP(_CFFI_OP_NOOP, 47) },
  { "nvp_size", offsetof(nvpair_t, nvp_size),
                sizeof(((nvpair_t *)0)->nvp_size),
                _CFFI_OP(_CFFI_OP_NOOP, 47) },
  { "nvp_name_sz", offsetof(nvpair_t, nvp_name_sz),
                   sizeof(((nvpair_t *)0)->nvp_name_sz),
                   _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "nvp_reserve", offsetof(nvpair_t, nvp_reserve),
                   sizeof(((nvpair_t *)0)->nvp_reserve),
                   _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "nvp_value_elem", offsetof(nvpair_t, nvp_value_elem),
                      sizeof(((nvpair_t *)0)->nvp_value_elem),
                      _CFFI_OP(_CFFI_OP_NOOP, 47) },
  { "nvp_type", offsetof(nvpair_t, nvp_type),
                sizeof(((nvpair_t *)0)->nvp_type),
                _CFFI_OP(_CFFI_OP_NOOP, 45) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "nvlist", 48, _CFFI_F_CHECK_FIELDS,
    sizeof(nvlist_t), offsetof(struct _cffi_align__nvlist_t, y), 0, 5 },
  { "nvpair", 49, _CFFI_F_CHECK_FIELDS,
    sizeof(nvpair_t), offsetof(struct _cffi_align__nvpair_t, y), 5, 5 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$boolean_t", 30, _cffi_prim_int(sizeof(boolean_t), ((boolean_t)-1) <= 0),
    "_B_FALSE,_B_TRUE" },
  { "$data_type_t", 45, _cffi_prim_int(sizeof(data_type_t), ((data_type_t)-1) <= 0),
    "DATA_TYPE_UNKNOWN,DATA_TYPE_BOOLEAN,DATA_TYPE_BYTE,DATA_TYPE_INT16,DATA_TYPE_UINT16,DATA_TYPE_INT32,DATA_TYPE_UINT32,DATA_TYPE_INT64,DATA_TYPE_UINT64,DATA_TYPE_STRING,DATA_TYPE_BYTE_ARRAY,DATA_TYPE_INT16_ARRAY,DATA_TYPE_UINT16_ARRAY,DATA_TYPE_INT32_ARRAY,DATA_TYPE_UINT32_ARRAY,DATA_TYPE_INT64_ARRAY,DATA_TYPE_UINT64_ARRAY,DATA_TYPE_STRING_ARRAY,DATA_TYPE_HRTIME,DATA_TYPE_NVLIST,DATA_TYPE_NVLIST_ARRAY,DATA_TYPE_BOOLEAN_VALUE,DATA_TYPE_INT8,DATA_TYPE_UINT8,DATA_TYPE_BOOLEAN_ARRAY,DATA_TYPE_INT8_ARRAY,DATA_TYPE_UINT8_ARRAY,DATA_TYPE_DOUBLE" },
  { "$f_attr_t", 4, _cffi_prim_int(sizeof(f_attr_t), ((f_attr_t)-1) <= 0),
    "F_ATTR_INVAL,F_ARCHIVE,F_HIDDEN,F_READONLY,F_SYSTEM,F_APPENDONLY,F_NODUMP,F_IMMUTABLE,F_AV_MODIFIED,F_OPAQUE,F_AV_SCANSTAMP,F_AV_QUARANTINED,F_NOUNLINK,F_CRTIME,F_OWNERSID,F_GROUPSID,F_FSID,F_REPARSE,F_GEN,F_OFFLINE,F_SPARSE,F_ATTR_ALL" },
  { "$xattr_view_t", 14, _cffi_prim_int(sizeof(xattr_view_t), ((xattr_view_t)-1) <= 0),
    "XATTR_VIEW_INVALID,XATTR_VIEW_READONLY,XATTR_VIEW_READWRITE,XATTR_VIEW_LAST" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "boolean_t", 30 },
  { "data_type_t", 45 },
  { "f_attr_t", 4 },
  { "nvlist_t", 48 },
  { "nvpair_t", 49 },
  { "uint_t", 24 },
  { "xattr_view_t", 14 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  70,  /* num_globals */
  2,  /* num_struct_unions */
  4,  /* num_enums */
  7,  /* num_typenames */
  NULL,  /* no includes */
  53,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__sysattr(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__sysattr(void) { return NULL; }
#  else
     init_sysattr(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__sysattr(void)
{
  return _cffi_init("_sysattr", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_sysattr(void)
{
  _cffi_init("_sysattr", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
